@@ -142,7 +142,12 @@ def test_read_mwa_read_cotter():
     mwa_uv = UVData()
     cotter_uv = UVData()
     # cotter data has cable correction and is unphased
-    mwa_uv.read(filelist[0:2], correct_cable_len=True)
+    mwa_uv.read(
+        filelist[0:2],
+        correct_cable_len=True,
+        remove_dig_gains=False,
+        remove_coarse_band=False,
+    )
     cotter_uv.read(filelist[6])
     # cotter doesn't record the auto xy polarizations
     # due to a possible bug in cotter, the auto yx polarizations are conjugated
@@ -247,14 +252,10 @@ def test_read_mwa_flags():
         "some coarse channel files were not submitted",
     ]
     with uvtest.check_warnings(UserWarning, messages):
-        mwa_uv.read(subfiles)
+        mwa_uv.read(subfiles, use_cotter_flags=False)
 
     del mwa_uv
-    mwa_uv = UVData()
-    with pytest.raises(NotImplementedError) as cm:
-        mwa_uv.read(subfiles, use_cotter_flags=True)
-    assert str(cm.value).startswith("reading in cotter flag files")
-    del mwa_uv
+
     mwa_uv = UVData()
     with pytest.raises(ValueError) as cm:
         mwa_uv.read(subfiles[0:2], use_cotter_flags=True)
@@ -418,8 +419,9 @@ def test_flag_nsample_basic():
     bad = uv.select(antenna_nums=bad_ants, inplace=False)
     good = uv.select(antenna_nums=good_ants, inplace=False)
     assert np.all(bad.flag_array)
+    # TODO: Spw axis to be collapsed in future release
     good.flag_array = good.flag_array.reshape(
-        (good.Ntimes, good.Nbls, good.Nspws, good.Nfreqs, good.Npols)
+        (good.Ntimes, good.Nbls, 1, good.Nfreqs, good.Npols)
     )
     # good ants should be flagged except for the first time and second freq,
     # and for the second time and first freq
@@ -429,8 +431,9 @@ def test_flag_nsample_basic():
     assert np.all(good.flag_array[0, :, :, 0, :])
     assert np.all(good.flag_array[-1, :, :, 1, :])
     # check that nsample array is filled properly
+    # TODO: Spw axis to be collapsed in future release
     uv.nsample_array = uv.nsample_array.reshape(
-        (uv.Ntimes, uv.Nbls, uv.Nspws, uv.Nfreqs, uv.Npols)
+        (uv.Ntimes, uv.Nbls, 1, uv.Nfreqs, uv.Npols)
     )
     assert np.all(uv.nsample_array[1:-1, :, :, :, :] == 0.0)
     assert np.all(uv.nsample_array[0, :, :, 1, :] == 1.0)
@@ -478,7 +481,8 @@ def test_flag_start_flag(flag_file_init):
         flag_dc_offset=False,
     )
 
-    reshape = [uv.Ntimes, uv.Nbls, uv.Nspws, uv.Nfreqs, uv.Npols]
+    # TODO: Spw axis to be collapsed in future release
+    reshape = [uv.Ntimes, uv.Nbls, 1, uv.Nfreqs, uv.Npols]
     time_inds = [0, 1, -1, -2]
     assert np.all(
         uv.flag_array.reshape(reshape)[time_inds, :, :, :, :]
@@ -591,3 +595,118 @@ def test_invalid_precision_errors():
         uv.read_mwa_corr_fits(filelist[0:2], nsample_array_dtype=np.complex128)
 
     return
+
+
+@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+def test_remove_dig_gains():
+    """Test digital gain removal."""
+    uv1 = UVData()
+    uv1.read(filelist[0:2])
+
+    uv2 = UVData()
+    uv2.read(filelist[0:2], remove_dig_gains=False)
+
+    with fits.open(filelist[0]) as meta:
+        meta_tbl = meta[1].data
+        antenna_numbers = meta_tbl["Antenna"][1::2]
+        dig_gains = meta_tbl["Gains"][1::2, :].astype(np.float64) / 64
+    reordered_inds = antenna_numbers.argsort()
+    dig_gains = dig_gains[reordered_inds, :]
+    dig_gains = dig_gains[:, np.array([23])]
+    dig_gains = np.repeat(dig_gains, 1, axis=1)
+    dig_gains1 = dig_gains[uv2.ant_1_array, :]
+    dig_gains2 = dig_gains[uv2.ant_2_array, :]
+    dig_gains1 = dig_gains1[:, :, np.newaxis, np.newaxis]
+    dig_gains2 = dig_gains2[:, :, np.newaxis, np.newaxis]
+    uv2.data_array = uv2.data_array / (dig_gains1 * dig_gains2)
+
+    assert uv1 == uv2
+
+
+@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+def test_remove_coarse_band():
+    """Test coarse band removal."""
+    uv1 = UVData()
+    uv1.read(filelist[0:2])
+
+    uv2 = UVData()
+    uv2.read(filelist[0:2], remove_coarse_band=False)
+
+    with open(DATA_PATH + "/mwa_config_data/MWA_rev_cb_10khz_doubles.txt", "r") as f:
+        cb = f.read().splitlines()
+    cb_array = np.array(cb).astype(np.float64)
+    cb_array = cb_array.reshape(32, 4)
+    cb_array = np.average(cb_array, axis=1)
+    cb_array = cb_array[0]
+
+    uv2.data_array = uv2.data_array.swapaxes(2, 3)
+    uv2.data_array = uv2.data_array / cb_array
+    uv2.data_array = uv2.data_array.swapaxes(2, 3)
+
+    assert uv1 == uv2
+
+
+def test_cotter_flags():
+    """Test using cotter flags"""
+    uv = UVData()
+    files = filelist[0:2]
+    files.append(filelist[3])
+    messages = [
+        "telescope_location is not set.",
+        "some coarse channel files were not submitted",
+        "coarse channel, start time, and end time flagging will default",
+    ]
+    with uvtest.check_warnings(UserWarning, messages):
+        uv.read_mwa_corr_fits(files, flag_init=False)
+
+    with fits.open(filelist[3]) as aoflags:
+        flags = aoflags[1].data.field("FLAGS")
+    flags = flags[:, np.newaxis, :, np.newaxis]
+    flags = np.repeat(flags, 4, axis=3)
+
+    assert np.all(uv.flag_array == flags)
+
+
+@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
+@pytest.mark.filterwarnings(
+    "ignore:coarse channels are not contiguous for this observation"
+)
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:coarse channel, start time, and end time flagging")
+def test_cotter_flags_multiple(tmp_path):
+    """Test cotter flags with multiple coarse bands"""
+    mod_mini_6 = str(tmp_path / "mini_gpubox06_01.fits")
+    with fits.open(filelist[2]) as mini6:
+        mini6[1].header["time"] = 1447698337
+        mini6.writeto(mod_mini_6)
+    files = filelist[0:2] + filelist[3:5]
+    files.append(mod_mini_6)
+
+    uv = UVData()
+    uv.read_mwa_corr_fits(files, flag_init=False)
+
+    with fits.open(filelist[3]) as aoflags:
+        flags1 = aoflags[1].data.field("FLAGS")
+    with fits.open(filelist[4]) as aoflags:
+        flags2 = aoflags[1].data.field("FLAGS")
+    flags = np.array([flags2[:, 0], flags1[:, 0]])
+    flags = np.transpose(flags)
+    flags = flags[:, np.newaxis, :, np.newaxis]
+    flags = np.repeat(flags, 4, axis=3)
+
+    assert np.all(uv.flag_array == flags)
+
+
+@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:coarse channel, start time, and end time flagging")
+def test_mismatch_flags():
+    """Break by submitting flag and gpubox files from different coarse bands."""
+    uv = UVData()
+    files = filelist[0:2]
+    files.append(filelist[4])
+    with pytest.raises(ValueError) as cm:
+        uv.read(files)
+    assert str(cm.value).startswith("flag file coarse bands do not match")
